//=====================================================================
//	     
//	.
//=====================================================================

//---------------------------------------------------------------------
#property copyright 	"Dima S., 2010 ."
#property link      	"dimascub@mail.com"
//---------------------------------------------------------------------

//=====================================================================
//	 :
//=====================================================================
#include	<Object.mqh>
#include	<Arrays\List.mqh>
#include	<Arrays\ArrayObj.mqh>
#include	<BalanceSlope.mqh>
//---------------------------------------------------------------------

//=====================================================================
//	      :
//=====================================================================
class TCustomCriterion : public CObject
  {
protected:
   int               criterion_level;                                 //   ( 0 - ,   .. )

public:
   int               GetCriterionLevel();
   virtual double    GetCriterion();                                  //    
  };
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	  
//---------------------------------------------------------------------
int  TCustomCriterion::GetCriterionLevel()
  {
   return(this.criterion_level);
  }
//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double  TCustomCriterion::GetCriterion()
  {
   return(0.0);
  }
//---------------------------------------------------------------------

//=====================================================================
//	   ""   :
//	  ( Criterion ):
//=====================================================================
class TSimpleCriterion : public TCustomCriterion
  {
protected:
   ENUM_STATISTICS   stat_param_type;

public:
   ENUM_STATISTICS   GetCriterionType();                              //     

public:
   virtual double    GetCriterion();                                  //    
                     TSimpleCriterion(ENUM_STATISTICS _stat);
  };
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
TSimpleCriterion::TSimpleCriterion(ENUM_STATISTICS _stat)
   :
stat_param_type(_stat)
  {
   this.criterion_level=0;
  }
//---------------------------------------------------------------------
//	    :
//---------------------------------------------------------------------
ENUM_STATISTICS  TSimpleCriterion::GetCriterionType()
  {
   return(this.stat_param_type);
  }
//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double  TSimpleCriterion::GetCriterion()
  {
   return(TesterStatistics(this.stat_param_type));
  }
//---------------------------------------------------------------------

//=====================================================================
//	   ""   
//	  ( 1 / Criterion ):
//=====================================================================
class TSimpleDivCriterion : public TSimpleCriterion
  {
public:
   virtual double    GetCriterion();                                  //    
                     TSimpleDivCriterion(ENUM_STATISTICS _stat);
  };
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
TSimpleDivCriterion::TSimpleDivCriterion(ENUM_STATISTICS _stat)
   :
TSimpleCriterion(_stat)
  {
  }
//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double  TSimpleDivCriterion::GetCriterion()
  {
   double   temp=TesterStatistics(this.stat_param_type);
   if(temp>0.0)
     {
      return(1.0/temp);
     }
   return(0.0);
  }
//---------------------------------------------------------------------

//=====================================================================
//	   ""   
//	  ( Koeff * Criterion ):
//=====================================================================
class TSimpleMulCriterion : public TSimpleCriterion
  {
private:
   double            koeff;

public:
   virtual double    GetCriterion();                                   //    
                     TSimpleMulCriterion(ENUM_STATISTICS _stat,double _koeff);
  };
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
TSimpleMulCriterion::TSimpleMulCriterion(ENUM_STATISTICS _stat,double _koeff)
   :
TSimpleCriterion(_stat),
koeff(_koeff)
  {
  }
//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double  TSimpleMulCriterion::GetCriterion()
  {
   return(koeff*TesterStatistics(this.stat_param_type));
  }
//---------------------------------------------------------------------

//=====================================================================
//	   ""   
//	  ( Koeff - Criterion ):
//=====================================================================
class TSimpleSubCriterion : public TSimpleCriterion
  {
private:
   double            koeff;

public:
   virtual double    GetCriterion();                                  //    
                     TSimpleSubCriterion(ENUM_STATISTICS _stat,double _koeff);
  };
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
TSimpleSubCriterion::TSimpleSubCriterion(ENUM_STATISTICS _stat,double _koeff)
   :
TSimpleCriterion(_stat),
koeff(_koeff)
  {
  }
//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double  TSimpleSubCriterion::GetCriterion()
  {
   return(koeff-TesterStatistics(this.stat_param_type));
  }
//---------------------------------------------------------------------

//=====================================================================
//	   ""   
//	  Koeff1 * ( Koeff2 - Criterion ):
//=====================================================================
class TSimpleMulSubCriterion : public TSimpleCriterion
  {
private:
   double            koeff1;
   double            koeff2;

public:
   virtual double    GetCriterion();                                  //    
                     TSimpleMulSubCriterion(ENUM_STATISTICS _stat,double _koeff1,double _koeff2);
  };
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
TSimpleMulSubCriterion::TSimpleMulSubCriterion(ENUM_STATISTICS _stat,double _koeff1,double _koeff2)
   :
TSimpleCriterion(_stat),
koeff1(_koeff1),
koeff2(_koeff2)
  {
  }
//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double  TSimpleMulSubCriterion::GetCriterion()
  {
   return(koeff1 *(koeff2-TesterStatistics(this.stat_param_type)));
  }
//---------------------------------------------------------------------

//=====================================================================
//	   ""   :
//	  ( Criterion )   :
//=====================================================================
class TSimpleMinCriterion : public TSimpleCriterion
  {
   double            min_stat_param;

public:
   virtual double    GetCriterion();                                  //    
                     TSimpleMinCriterion(ENUM_STATISTICS _stat,double _min);
  };
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
TSimpleMinCriterion::TSimpleMinCriterion(ENUM_STATISTICS _stat,double _min)
   :
TSimpleCriterion(_stat),
min_stat_param(_min)
  {
  }
//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double  TSimpleMinCriterion::GetCriterion()
  {
   double   temp=TesterStatistics(this.stat_param_type);
   if(temp<this.min_stat_param)
     {
      return(-1.0);
     }
   return(temp);
  }
//---------------------------------------------------------------------

//=====================================================================
//	   ""   :
//	  ( Criterion )   :
//=====================================================================
class TSimpleMaxCriterion : public TSimpleCriterion
  {
   double            max_stat_param;

public:
   virtual double    GetCriterion();                                  //    
                     TSimpleMaxCriterion(ENUM_STATISTICS _stat,double _max);
  };
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
TSimpleMaxCriterion::TSimpleMaxCriterion(ENUM_STATISTICS _stat,double _max)
   :
TSimpleCriterion(_stat),
max_stat_param(_max)
  {
  }
//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double  TSimpleMaxCriterion::GetCriterion()
  {
   double   temp=TesterStatistics(this.stat_param_type);
   if(temp>this.max_stat_param)
     {
      return(-1.0);
     }
   return(temp);
  }
//---------------------------------------------------------------------

//	STAT_INITIAL_DEPOSIT -    - double
//	STAT_WITHDRAWAL -      - double
//	STAT_PROFIT -     ,  STAT_GROSS_PROFIT  STAT_GROSS_LOSS (STAT_GROSS_LOSS     ) - double
//	STAT_GROSS_PROFIT -  ,    () .      - double
//	STAT_GROSS_LOSS -  ,    () .      - double
//	STAT_MAX_PROFITTRADE -         .      - double
//	STAT_MAX_LOSSTRADE -         .      - double
//	STAT_CONPROFITMAX -      .      - double
//	STAT_CONPROFITMAX_TRADES -  ,  STAT_CONPROFITMAX (     ) - int
//	STAT_MAX_CONWINS -         - double
//	STAT_MAX_CONPROFIT_TRADES -          STAT_MAX_CONWINS - int
//	STAT_CONLOSSMAX -      .      - double
//	STAT_CONLOSSMAX_TRADES -  ,  STAT_CONLOSSMAX (     ) - int
//	STAT_MAX_CONLOSSES -         - double
//	STAT_MAX_CONLOSS_TRADES -          STAT_MAX_CONLOSSES - int
//	STAT_BALANCEMIN -    - double
//	STAT_BALANCE_DD -     .        ,   . - double
//	STAT_BALANCEDD_PERCENT -    ,           (STAT_BALANCE_DD). - double
//	STAT_BALANCE_DDREL_PERCENT -     .        ,        .    - double
//	STAT_BALANCE_DD_RELATIVE -    ,           (STAT_BALANCE_DDREL_PERCENT). - double
//	STAT_EQUITYMIN -     - double
//	STAT_EQUITY_DD -     .        ,   . - double
//	STAT_EQUITYDD_PERCENT -    ,           (STAT_EQUITY_DD). - double
//	STAT_EQUITY_DDREL_PERCENT -     .        ,        .    - double
//	STAT_EQUITY_DD_RELATIVE -    ,           (STAT_EQUITY_DDREL_PERCENT). - double
//	STAT_EXPECTED_PAYOFF -    - double
//	STAT_PROFIT_FACTOR -    STAT_GROSS_PROFIT/STAT_GROSS_LOSS.  STAT_GROSS_LOSS=0,     DBL_MAX - double
//	STAT_RECOVERY_FACTOR -     STAT_PROFIT/STAT_BALANCE_DD - double
//	STAT_SHARPE_RATIO -   - double
//	STAT_MAX_MARGINLEVEL -      - double
//	STAT_CUSTOM_ONTESTER -     ,   OnTester() - double
//	STAT_DEALS -    - int
//	STAT_TRADES -   - int
//	STAT_PROFIT_TRADES -   - int
//	STAT_LOSS_TRADES -   - int
//	STAT_SHORT_TRADES -   - int
//	STAT_LONG_TRADES -   - int
//	STAT_PROFIT_SHORTTRADES -    - int
//	STAT_PROFIT_LONGTRADES -    - int
//	STAT_PROFITTRADES_AVGCON -      - int
//	STAT_LOSSTRADES_AVGCON -      - int

//=====================================================================
//	       
//	  :
//=====================================================================
class TBalanceSlopeCriterion : public TCustomCriterion
  {
private:
   double            scale;

private:
   TBalanceSlope    *balance_Ptr;

public:
                     TBalanceSlopeCriterion(string _symbol,double _sc=1000.0);
   virtual double    GetCriterion();                                  //    
  };
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	:
//---------------------------------------------------------------------
TBalanceSlopeCriterion::TBalanceSlopeCriterion(string _symbol,double _sc)
   :
scale(_sc)
  {
   balance_Ptr=new TBalanceSlope(_symbol);
  }
//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double  TBalanceSlopeCriterion::GetCriterion()
  {
//	    :
   double   current_slope=1000.0*this.balance_Ptr.CalcSlope();

//	  :
   if(current_slope<0.0)
     {
      return(-1.0);
     }

   double   temp=this.balance_Ptr.GetCurrentSKO();
   if(temp>0.0)
     {
      return(this.scale/temp);
     }

   return(0.0);
  }
//---------------------------------------------------------------------

//=====================================================================
//	       
//	    (  ):
//=====================================================================
class TTSSFCriterion : public TCustomCriterion
  {
public:
   virtual double    GetCriterion();                                  //    
  };
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double  TTSSFCriterion::GetCriterion()
  {
   double   avg_win=TesterStatistics(STAT_GROSS_PROFIT)/TesterStatistics(STAT_PROFIT_TRADES);
   double   avg_loss = -TesterStatistics( STAT_GROSS_LOSS ) / TesterStatistics( STAT_LOSS_TRADES );
   double   win_perc = 100.0 * TesterStatistics( STAT_PROFIT_TRADES ) / TesterStatistics( STAT_TRADES );

//	  :
   double   teor=(110.0-win_perc)/(win_perc-10.0)+1.0;

//	  :
   double   real=avg_win/avg_loss;

//	:
   double   tssf=real/teor;

   return(tssf);
  }
//---------------------------------------------------------------------

//=====================================================================
//	      :
//=====================================================================
class TCustomCriterionArray : public CArrayObj
  {
public:
   virtual double    GetCriterion();                                  //    
  };
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//	   :
//---------------------------------------------------------------------
double  TCustomCriterionArray::GetCriterion()
  {
   double   temp=1.0;
   int         count=this.Total();
   if(count==0)
     {
      return(0.0);
     }
   for(int i=0; i<count; i++)
     {
      temp*=(( TCustomCriterion*)(this.At(i))).GetCriterion();
      if(temp<0.0)
        {
         return(temp);
        }
     }

   return(temp);
  }
//---------------------------------------------------------------------
